@@ -25,7 +25,7 @@ static char rcsid[] = "$Id: domainname.c,v 2.2 1995/01/24 12:24:45 swen Exp $" ;
 void
 usage(void)
 {
-  puts("usage: domainname [nisdomain]") ;
+  puts("Usage: domainname [nisdomain]") ;
   exit(1) ;
 }
 
@@ -34,7 +34,7 @@ main(int argc, char *argv[])
 {
   char *name, *tmp ;
   
-  if (argc > 2)
+  if (argc > 2 || (argc == 2 && argv[1][0] == '-'))
       usage();
 
   name = (char *) malloc(DOMAINNAMELEN) ;
