@@ -25,27 +25,27 @@
 #include <rpc/pmap_clnt.h> /* for pmap_unset */
 #include "ourhdr.h"
 #include "ypbind.h"
-#if USE_IPC
 #include <sys/ipc.h>
 #include <sys/sem.h>
 #include <sys/shm.h>
-#endif
 
+#if 0
 extern int putenv(const char *);
+#endif
 
 extern SVCXPRT *udptransp;
 extern domainname mydomain;
 extern int broken_server;
-extern int debug;
+extern int debug, use_ipc, use_mmap;
 extern char **Argv;
 extern int Argc;
 
+extern CLIENT * clntudp_create(struct sockaddr_in *, u_long, u_long,
+                               struct timeval, int *);
+
 struct binding *ypbindlist;
-#if USE_IPC
 int semid = -1;
-#else
 int lockfd;
-#endif
 domainname askdomain;
 volatile int hangup;
 
@@ -72,17 +72,17 @@ start_slave(void)
   if (0 != sigaction(SIGHUP, &sact, NULL))
     log_ret("Could not install signal handler for SIGHUP");
   
-  for(;;)
-  {
-    hangup = 0;
-    init_binding();
-  
-    for(;!hangup;)
-      {
-        check_binding();
-        sleep(PING_INTERVAL);
-      }
-  }
+  for (;;)
+    {
+      hangup = 0;
+      init_binding();
+      
+      while (!hangup)
+        {
+          check_binding();
+          sleep(PING_INTERVAL);
+        }
+    }
 }
 
 void
@@ -91,20 +91,23 @@ init_binding(void)
   struct binding ypdb;
 
   write_lock_binding();
-  bzero(ypbindlist, sizeof(struct binding) * _MAXDOMAIN);
-#if !MMAP_SHARED_OK
-  lseek(lockfd, 0, SEEK_SET);
-  writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN);
-#endif
+  memset(ypbindlist, 0, sizeof(struct binding) * _MAXDOMAIN);
+  if (!use_mmap)
+    {
+      lseek(lockfd, 0, SEEK_SET);
+      if (writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN) <= 0)
+        log_sys("cannot write ypbindlist");
+    }
   un_lock_write_binding();
 
   memset((void *)&ypdb, 0, sizeof(struct binding));
   strncpy(ypdb.domain, mydomain, YPMAXDOMAIN);
   ypdb.is_bound = TRUE; /* always bind to our NIS domain */
+  ypdb.active = 0;
   ypdb.lockfd = -1;
-  ypdb.client_handle = NULL;
+  ypdb.server[0].client_handle = NULL;
   ypdb.is_alive = FALSE;
-  ypdb.use_broadcast = TRUE; /* default is using broadcast */
+  ypdb.server[0].use_broadcast = TRUE; /* default is using broadcast */
   update_entry(&ypdb);
   parse_config_file(_PATH_YPCONF);
   return;
@@ -115,8 +118,8 @@ check_binding(void)
 {
   struct binding ypdb;
   time_t t;
-  int i;
-
+  int i, j;
+  
   time(&t);
   for (i = 0; i < _MAXDOMAIN; i++)
     {
@@ -125,28 +128,40 @@ check_binding(void)
       un_lock_read_binding();
       if (ypdb.is_bound)
         {
-          if (ypdb.is_alive && ypdb.next_check > t)
+          if (ypdb.is_alive)
             {
-              if (debug)
-                  log_msg("pinging server %s, port %d",
-                          inet_ntoa(ypdb.server_addr),
-                          ntohs(ypdb.server_port));
-              ping_server(&ypdb);
+              int active;
+              for (j = 0; j < _MAXSERVER; j++)
+                {
+                  active = (ypdb.active + j) % _MAXSERVER;
+                  if (!ypdb.server[active].filled) continue;
+                  if (debug)
+                    log_msg("pinging server %s, port %d",
+                            inet_ntoa(ypdb.server[active].server_addr),
+                            ntohs(ypdb.server[active].server_port));
+                  ypdb.is_alive = FALSE;
+                  ping_server(&ypdb, active);
+                  if (ypdb.is_alive) break;
+                }
+              if (active != ypdb.active && ypdb.server[active].filled)
+                {
+                  ypdb.active=active;
+                  update_entry(&ypdb); 
+                }
             }
-          
-          if (!ypdb.is_alive || ypdb.next_check <= t)
+          if (!ypdb.is_alive)
             {
-              if (ypdb.use_broadcast)
+              if (ypdb.server[ypdb.active].use_broadcast)
                 {
                   if (debug)
-                      log_msg("broadcasting for domain %s", ypdb.domain);
+                    log_msg("broadcasting for domain %s", ypdb.domain);
                   broadcast(&ypdb);
                 }
               else
                 {
                   if (debug)
-                    log_msg("rebinding to server %s", ypdb.host->h_name);
-                  bindto_server(ypdb.domain, ypdb.host);
+                    log_msg("binding to server %s", ypdb.server[ypdb.active].host);
+                  bindto_server(ypdb.domain, ypdb.server[ypdb.active].host);
                 }
               continue;
             }
@@ -155,7 +170,8 @@ check_binding(void)
 }
 
 void
-add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct hostent *host, bool_t use_broadcast)
+add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep,
+           const char *host, bool_t use_broadcast)
 {
   struct binding entry;
 #if USE_BINDINGDIR
@@ -163,19 +179,33 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
   struct ypbind_resp ybr;
   pid_t lockpid;
   char path[MAXPATHLEN];
-  int fd, len, status;
+  int fd, len, status, active;
 #endif
   
   if (NULL == dom || (0 != get_entry(dom, &entry)))
     return;
-  
+
   if (!entry.is_bound)
     {
-      strncpy(entry.domain, dom, YPMAXDOMAIN);
+      entry.active = 0;
       entry.lockfd = -1;
-      entry.client_handle = NULL;
-      entry.is_bound = TRUE;
+      memset((void *)&entry.server, 0, sizeof(struct bound_server));
+      entry.is_alive = FALSE;
     }
+
+  /* find empty slot */
+  for (active = 0; active < _MAXSERVER; active++)
+
+    if (!entry.server[active].filled) break;
+  
+  active = active % _MAXSERVER;
+  if (use_broadcast) active =  _MAXSERVER - 1;
+
+  if (debug)
+    log_msg("add_server() domain: %s, host: %s, %sbroadcast, slot: %d",
+            dom, host ? host : "unknown", use_broadcast ? "" : "no",
+            active);
+  
   if (NULL != raddrp)
     {
       if (!broken_server &&
@@ -185,18 +215,23 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
 	  log_msg("Answer from %s on illegal port", inet_ntoa(raddrp->sin_addr));
 	  return;
 	}
-      memcpy(&entry.server_addr, &raddrp->sin_addr, sizeof entry.server_addr);
-      entry.server_port = raddrp->sin_port;
+      memcpy(&entry.server[active].server_addr, &raddrp->sin_addr,
+             sizeof entry.server[active].server_addr);
+      entry.server[active].server_port = raddrp->sin_port;
       entry.is_alive = TRUE;
     }
-  if (NULL != entry.client_handle)
-    clnt_destroy(entry.client_handle);
-  entry.client_handle = clnt_handlep;
+  if (NULL != entry.server[active].client_handle)
+    clnt_destroy(entry.server[active].client_handle);
+  entry.server[active].client_handle = clnt_handlep;
   if (NULL != host)
-    entry.host = host;
-  entry.next_check = time(NULL) + REBIND_INTERVAL;
-  entry.version = YPVERS;
-  entry.use_broadcast = use_broadcast;
+    {
+      if (entry.server[active].host) free(entry.server[active].host);
+      entry.server[active].host = strdup(host);
+    }
+  entry.server[active].version = YPVERS;
+  entry.server[active].use_broadcast = use_broadcast;
+  entry.server[active].filled = TRUE;
+  entry.active = active;
   
 #if USE_BINDINGDIR
   if (NULL != raddrp)
@@ -204,7 +239,7 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
       if (-1 != entry.lockfd)
         close(entry.lockfd);
       sprintf(path, "%s/%s.%ld", BINDINGDIR,
-              entry.domain, entry.version);
+              entry.domain, entry.server[active].version);
       if ((fd = open(path, O_CREAT | O_RDWR | O_TRUNC, FILE_MODE )) == -1)
         {
           if (-1 == mkdir(BINDINGDIR, DIR_MODE))
@@ -222,10 +257,10 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
         log_sys("set lock");
       
       
-          /*
-           * ok, if BINDINGDIR exists, and we can create the binding file, then
-           * write to it..
-           */
+      /*
+       * ok, if BINDINGDIR exists, and we can create the binding file, then
+       * write to it..
+       */
       entry.lockfd = fd;
       
       iov[0].iov_base = (caddr_t) &(udptransp->xp_port);
@@ -233,7 +268,7 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
       iov[1].iov_base = (caddr_t) &ybr;
       iov[1].iov_len = sizeof ybr;
       
-      bzero(&ybr, sizeof ybr);
+      memset(&ybr, 0, sizeof ybr);
       ybr.ypbind_status = YPBIND_SUCC_VAL;
       ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_addr = raddrp->sin_addr;
       ybr.ypbind_respbody.ypbind_bindinfo.ypbind_binding_port = raddrp->sin_port;
@@ -253,7 +288,7 @@ add_server(char *dom, struct sockaddr_in *raddrp, CLIENT *clnt_handlep, struct h
 }
 
 void
-bindto_server(char *ypdomain, struct hostent *host)
+bindto_server(char *ypdomain, char *server)
 {
   struct sockaddr_in server_addr;
   int sock;
@@ -263,10 +298,50 @@ bindto_server(char *ypdomain, struct hostent *host)
   CLIENT *clnt_handlep = NULL;
   int i = 0;
   
+  struct hostent *host;
+  static char *order = NULL;
+  int result;
+
   if (debug)
-    log_msg("bindto_server: domain %s, host %s", ypdomain, host->h_name);
+    log_msg("bindto_server: domain %s, host %s", ypdomain, server);
   
-  bzero((char *)&server_addr, sizeof server_addr);
+  /*
+   * FIXME: gethostbyname may in turn ask ypbind (entry in 
+   * /etc/host.conf)!!! 
+   * so much for shooting yourself in the foot.
+   * Using RESOLV_SERV_ORDER is a kludge, should use gethostent()
+   */
+  
+  if (!order)
+    order = strdup("RESOLV_SERV_ORDER=hosts");
+  result = putenv(order);
+  if (0 != result)
+    log_ret("putenv failed");
+  host = gethostbyname(server);
+  if (NULL == host)
+    {
+      switch (h_errno)
+        {
+        case HOST_NOT_FOUND:
+          log_msg("Unknown host: %s", server);
+          break;
+        case TRY_AGAIN:
+          log_msg("Host name lookup failure");
+          break;
+        case NO_DATA:
+          log_msg("No address associated with name: %s", server);
+          break;
+        case NO_RECOVERY:
+          log_msg("Unknown server error");
+          break;
+        default:
+          log_ret("gethostbyname: Unknown error");
+          break;
+        }
+      return;
+    }
+  
+  memset((char *)&server_addr, 0, sizeof server_addr);
   server_addr.sin_family = host->h_addrtype;
   server_addr.sin_port = htons(0);
   sock = RPC_ANYSOCK;
@@ -280,10 +355,10 @@ bindto_server(char *ypdomain, struct hostent *host)
       clnt_handlep = clntudp_create(&server_addr, YPPROG, YPVERS,
                                     timeout, &sock);
       if (NULL != clnt_handlep)
-          break;
+        break;
       i++;
     }
-
+  
   if (NULL == clnt_handlep)
     {
       log_msg("clnt_create for server %s failed", host->h_name);
@@ -293,7 +368,7 @@ bindto_server(char *ypdomain, struct hostent *host)
   timeout.tv_sec = 5;
   timeout.tv_usec = 0;
   status = clnt_call(clnt_handlep, YPPROC_DOMAIN,
-                     (xdrproc_t) xdr_domainname, ypdomain,
+                     (xdrproc_t) xdr_domainname, &ypdomain,
                      (xdrproc_t) xdr_bool, &out,
                      timeout);
   if (RPC_SUCCESS != status)
@@ -307,37 +382,41 @@ bindto_server(char *ypdomain, struct hostent *host)
       clnt_destroy(clnt_handlep);
     }
   else
-    add_server(ypdomain, &server_addr, clnt_handlep, host, FALSE);
-
+    add_server(ypdomain, &server_addr, clnt_handlep, host->h_name, FALSE);
+  
   return;
 }
 
 void
-ping_server(struct binding *ypdb)
+ping_server(struct binding *ypdb, int active)
 {
   int status;
+  bool_t out;
   struct timeval timeout;
-  char *nothing;
   
-  if (NULL != ypdb->client_handle)
+  if (NULL != ypdb->server[active].client_handle)
     {
-      bzero((char *)&nothing, sizeof nothing);
+      char *domainname = ypdb->domain;
       timeout.tv_sec = 2;
       timeout.tv_usec = 0;
-      status = clnt_call(ypdb->client_handle, YPPROC_NULL,
-                         (xdrproc_t) xdr_void, nothing,
-                         (xdrproc_t) xdr_void, &nothing,
+      status = clnt_call(ypdb->server[active].client_handle, YPPROC_DOMAIN,
+                         (xdrproc_t) xdr_domainname, &domainname,
+                         (xdrproc_t) xdr_bool, &out,
                          timeout);
       
-      if (RPC_SUCCESS != status)
+      if ((RPC_SUCCESS != status) || (TRUE != out))
         {
-          log_msg(clnt_sperror(ypdb->client_handle, ypdb->host->h_name));
-	  clnt_destroy(ypdb->client_handle);
-          ypdb->client_handle = NULL;
+          RPC_SUCCESS != status
+	    ? log_msg(clnt_sperror(ypdb->server[active].client_handle,
+	                           ypdb->server[active].host ? ypdb->server[active].host : "unknown"))
+            : log_msg("domain %s not served by %s",
+                      ypdb->domain, ypdb->server[active].host ? ypdb->server[active].host : "unknown") ;
+          clnt_destroy(ypdb->server[active].client_handle);
+          ypdb->server[active].client_handle = NULL;
           ypdb->is_alive = FALSE;
-          ypdb->next_check = time(NULL);
-          update_entry(ypdb);
         }
+      else
+        ypdb->is_alive=TRUE;
     }
   return;
 }
@@ -384,7 +463,7 @@ broadcast(struct binding *ypdb)
       /* update global variable for eachresult */
   askdomain = ypdb->domain;
   status = clnt_broadcast(YPPROG, YPVERS, YPPROC_DOMAIN_NONACK,
-                          (xdrproc_t) xdr_domainname, askdomain,
+                          (xdrproc_t) xdr_domainname, (void *) &askdomain,
                           (xdrproc_t) xdr_bool, (void *)&out,
                           (resultproc_t) eachresult);
   if (RPC_SUCCESS != status)
@@ -395,7 +474,7 @@ broadcast(struct binding *ypdb)
           close(ypdb->lockfd);
           ypdb->lockfd = -1;
           sprintf(path, "%s/%s.%ld", BINDINGDIR,
-                  ypdb->domain, ypdb->version);
+                  ypdb->domain, ypdb->server[ypdb->active].version);
           unlink(path);
         }
 #endif
@@ -443,7 +522,7 @@ parse_config_file(const char *path)
         {
           if (debug)
             log_msg("parsed domain %s server %s", tmpdomain, tmpserver);
-          check_config_entry(tmpdomain, tmpserver);
+          bindto_server(tmpdomain, tmpserver);
           continue;
         }
       count = sscanf(cp, "domain %s broadcast", tmpdomain);
@@ -459,7 +538,7 @@ parse_config_file(const char *path)
         {
           if (debug)
             log_msg("parsed ypserver %s", tmpserver);
-          check_config_entry(mydomain, tmpserver);
+          bindto_server(mydomain, tmpserver);
           continue;
         }
     }
@@ -467,115 +546,76 @@ parse_config_file(const char *path)
   return;
 }
 
-void
-check_config_entry(char *ypdomain, const char *ypserver)
-{
-  struct hostent *host;
-  static char *order = NULL;
-  int result;
-
-      /*
-       * FIXME: gethostbyname may in turn ask ypbind (entry in 
-       * /etc/host.conf)!!! 
-       * so much for shooting yourself in the foot.
-       * Using RESOLV_SERV_ORDER is a kludge, should use gethostent()
-       */
-
-  if (!order)
-    order = strdup("RESOLV_SERV_ORDER=hosts");
-  result = putenv(order);
-  if (0 != result)
-    log_ret("putenv failed");
-  if (debug)
-    log_msg("getenv: %s", getenv("RESOLV_SERV_ORDER"));
-  host = gethostbyname(ypserver);
-  if (NULL == host)
-    {
-      switch (h_errno)
-        {
-        case HOST_NOT_FOUND:
-          log_msg("Unknown host: %s", ypserver);
-          break;
-        case TRY_AGAIN:
-          log_msg("Host name lookup failure");
-          break;
-        case NO_DATA:
-          log_msg("No address associated with name: %s", ypserver);
-          break;
-        case NO_RECOVERY:
-          log_msg("Unknown server error");
-          break;
-        default:
-          log_ret("gethostbyname: Unknown error");
-          break;
-        }
-    }
-  else
-    bindto_server(ypdomain, host);
-
-  return;
-}
-
 void
 init_master_slave_communication(void)
 {
-#if USE_IPC
   int shmid;
   struct shmid_ds dummy;
-  ushort empty[2] = {0,0};
+  ushort empty [2] = {0,0};
+  union semun semarg;
 
-  if ((semid = semget(IPC_PRIVATE, 2, IPC_CREAT|SHM_R|SHM_W)) < 0)
-    log_sys("cannot create semaphore");
-
-
-  if ((shmid = shmget(IPC_PRIVATE, 
-                      sizeof(struct binding) * _MAXDOMAIN,
-                      IPC_CREAT|SHM_R|SHM_W)) < 0)
-    log_sys("cannot create shared memory segment");
-
-  ypbindlist = (struct binding *)shmat(shmid, NULL, 0);
-
-  /* mark it for deletion just in case we are not able to
-     handle it on our own. handles case of failed attachment
-     as well */
-  (void)shmctl(shmid, IPC_RMID, &dummy);  
-
-  if (!ypbindlist)
-    log_sys("cannot attach to shared memory segment");
-
-  memset(ypbindlist, 0, sizeof(struct binding) * _MAXDOMAIN);
-
-  semctl(semid, 0, SETALL, empty);
-
-#else
-
-  lockfd = open_lockfile();
-#if MMAP_SHARED_OK  
-  ypbindlist = (struct binding *) mmap(0, sizeof(struct binding) * _MAXDOMAIN,
-                                       (PROT_READ | PROT_WRITE), 
+  if (use_ipc)
+    {
+      if ((semid = semget(IPC_PRIVATE, 2, IPC_CREAT|SHM_R|SHM_W)) < 0)
+        log_sys("cannot create semaphore");
+      
+      
+      if ((shmid = shmget(IPC_PRIVATE, 
+                          sizeof(struct binding) * _MAXDOMAIN,
+                          IPC_CREAT|SHM_R|SHM_W)) < 0)
+        log_sys("cannot create shared memory segment");
+      
+      ypbindlist = (struct binding *)shmat(shmid, NULL, 0);
+      
+      /* mark it for deletion just in case we are not able to
+         handle it on our own. handles case of failed attachment
+         as well */
+      (void)shmctl(shmid, IPC_RMID, &dummy);  
+      
+      if (!ypbindlist)
+        log_sys("cannot attach to shared memory segment");
+      
+      memset(ypbindlist, 0, sizeof(struct binding) * _MAXDOMAIN);
+      
+      semarg.array = empty;
+      semctl(semid, 0, SETALL, semarg);
+    }
+  else
+    {
+      lockfd = open_lockfile();
+      if (use_mmap)
+        {
+          ypbindlist = (struct binding *) mmap(0,
+                                               sizeof(struct binding) * _MAXDOMAIN,
+                                               (PROT_READ | PROT_WRITE), 
 #if MMAP_SHARED_ANON_OK
-                                       (MAP_ANON | MAP_SHARED), -1, 0);
+                                               (MAP_ANON | MAP_SHARED), -1,
 #else  
-                                       MAP_SHARED, lockfd, 0);
+                                               MAP_SHARED, lockfd,
 #endif /* MMAP_SHARED_ANON_OK */
-  if ((caddr_t) -1 == (caddr_t) ypbindlist)
-    log_sys("cannot create shared region");
-#else
-  ypbindlist = (struct binding *) calloc(_MAXDOMAIN, sizeof(struct binding));
-  if (NULL == ypbindlist)
-    log_sys("cannot create shared region");
-  writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN);
-#endif /* MMAP_SHARED_OK */
-#endif
+                                               0);
+          if ((caddr_t) -1 == (caddr_t) ypbindlist)
+            log_sys("cannot create shared region");
+        }
+      else
+        {
+          ypbindlist = (struct binding *) calloc(_MAXDOMAIN, sizeof(struct binding));
+          if (NULL == ypbindlist)
+            log_sys("cannot create shared region");
+          if (writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN) <= 0)
+            log_sys("cannot write shared region");
+        }
+    }
 }
 
-#if USE_IPC
 void
 terminate_master_slave_communication(void)
 {
   if(semid != -1)
   {
-    (void)semctl(semid, 0, IPC_RMID, NULL);
+    union semun dummy;
+
+    (void)semctl(semid, 0, IPC_RMID, dummy);
     semid = -1;
   }
 }
@@ -586,92 +626,134 @@ void
 read_lock_binding(void)
 {
   struct sembuf sops[2];
-  int i;
-
-  sops[0].sem_num = 1; /* check if write semaphore is clear */
-  sops[0].sem_op = 0;
-  sops[0].sem_flg = IPC_NOWAIT;
-  sops[1].sem_num = 0; /* then signal reading */
-  sops[1].sem_op = 1;
-  sops[1].sem_flg = SEM_UNDO|IPC_NOWAIT;
-
-  for(i=0; i<IPC_LOCK_RETRY; i++)
-  {
-    if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
-      break;
-    sleep(1);
-  }
-  if(i==IPC_LOCK_RETRY)
-    log_sys("cannot create read lock");
+  int i, status;
+  
+  if (use_ipc)
+    {
+      sops[0].sem_num = 1; /* check if write semaphore is clear */
+      sops[0].sem_op = 0;
+      sops[0].sem_flg = IPC_NOWAIT;
+      sops[1].sem_num = 0; /* then signal reading */
+      sops[1].sem_op = 1;
+      sops[1].sem_flg = SEM_UNDO|IPC_NOWAIT;
+      
+      for(i=0; i<IPC_LOCK_RETRY; i++)
+        {
+          if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
+            break;
+          sleep(1);
+        }
+      if(i==IPC_LOCK_RETRY)
+        log_sys("cannot create read lock");
+    }
+  else
+    {
+      status = lock_reg(lockfd, F_SETLKW, F_RDLCK, 0, SEEK_SET, 0);
+      if (0 != status) 
+        log_sys("set read lock");
+      if (!use_mmap)
+        {
+          lseek(lockfd, 0, SEEK_SET);
+          if (readn(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN) < 0)
+            log_sys("cannot read ypbindlist");
+        }
+    }
+  return;
 }
 
 void 
 write_lock_binding(void)
 {
   struct sembuf sops[3];
-  int i;
-
-  sops[0].sem_num = 0; /* check if read semaphore is clear */
-  sops[0].sem_op = 0;
-  sops[0].sem_flg = IPC_NOWAIT;
-  sops[1].sem_num = 1; /* check if write semaphore is clear */
-  sops[1].sem_op = 0;
-  sops[1].sem_flg = IPC_NOWAIT;
-  sops[2].sem_num = 1; /* then signal writing */
-  sops[2].sem_op = 1;
-  sops[2].sem_flg = SEM_UNDO|IPC_NOWAIT;
-
-  for(i=0; i<IPC_LOCK_RETRY; i++)
-  {
-    if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
-      break;
-    sleep(1);
-  }
-  if(i==IPC_LOCK_RETRY)
-    log_sys("cannot create write lock");
+  int i, status;
+      
+  if (use_ipc)
+    {
+      sops[0].sem_num = 0; /* check if read semaphore is clear */
+      sops[0].sem_op = 0;
+      sops[0].sem_flg = IPC_NOWAIT;
+      sops[1].sem_num = 1; /* check if write semaphore is clear */
+      sops[1].sem_op = 0;
+      sops[1].sem_flg = IPC_NOWAIT;
+      sops[2].sem_num = 1; /* then signal writing */
+      sops[2].sem_op = 1;
+      sops[2].sem_flg = SEM_UNDO|IPC_NOWAIT;
+      
+      for(i=0; i<IPC_LOCK_RETRY; i++)
+        {
+          if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
+            break;
+          sleep(1);
+        }
+      if(i==IPC_LOCK_RETRY)
+        log_sys("cannot create write lock");
+    }
+  else
+    {
+      status = lock_reg(lockfd, F_SETLKW, F_WRLCK, 0, SEEK_SET, 0); 
+      if (0 != status) 
+        log_sys("set write lock");
+    }
+  return;
 }
 
-void 
+
+void
 un_lock_read_binding(void)
 {
   struct sembuf sops[1];
   int i;
-
-  sops[0].sem_num = 0; /* clear read semaphore */
-  sops[0].sem_op = -1;
-  sops[0].sem_flg = SEM_UNDO|IPC_NOWAIT;
-
-  for(i=0; i<IPC_LOCK_RETRY; i++)
-  {
-    if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
-      break;
-    sleep(1);
-  }
-  if(i==IPC_LOCK_RETRY)
-    log_sys("error unlocking");
+  
+  if (use_ipc)
+    {
+      sops[0].sem_num = 0; /* clear read semaphore */
+      sops[0].sem_op = -1;
+      sops[0].sem_flg = SEM_UNDO|IPC_NOWAIT;
+      
+      for(i=0; i<IPC_LOCK_RETRY; i++)
+        {
+          if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
+            break;
+          sleep(1);
+        }
+      if(i==IPC_LOCK_RETRY)
+        log_sys("error unlocking");
+    }
+  else
+    un_lock_write_binding();
+  return;
 }
 
 void 
 un_lock_write_binding(void)
 {
   struct sembuf sops[1];
-  int i;
-
-  sops[0].sem_num = 1; /* clear write semaphore */
-  sops[0].sem_op = -1;
-  sops[0].sem_flg = SEM_UNDO|IPC_NOWAIT;
-
-  for(i=0; i<IPC_LOCK_RETRY; i++)
-  {
-    if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
-      break;
-    sleep(1);
-  }
-  if(i==IPC_LOCK_RETRY)
-    log_sys("error unlocking");
+  int i, status;
+  
+  if (use_ipc)
+    {
+      sops[0].sem_num = 1; /* clear write semaphore */
+      sops[0].sem_op = -1;
+      sops[0].sem_flg = SEM_UNDO|IPC_NOWAIT;
+      
+      for(i=0; i<IPC_LOCK_RETRY; i++)
+        {
+          if (!semop(semid, sops, sizeof(sops)/sizeof(struct sembuf)))
+            break;
+          sleep(1);
+        }
+      if(i==IPC_LOCK_RETRY)
+        log_sys("error unlocking");
+    }
+  else
+    {
+      status = lock_reg(lockfd, F_SETLKW, F_UNLCK, 0, SEEK_SET, 0);
+      if (0 != status)
+        log_sys("unlock");
+    }
+  return;
 }
 
-#else
 
 int
 open_lockfile(void)
@@ -696,50 +778,6 @@ open_lockfile(void)
   return fd;
 }
 
-void 
-read_lock_binding(void)
-{
-  int status;
-
-  status = lock_reg(lockfd, F_SETLKW, F_RDLCK, 0, SEEK_SET, 0);
-  if (0 != status) 
-    log_sys("set read lock");
-#if !MMAP_SHARED_OK
-  lseek(lockfd, 0, SEEK_SET);
-  readn(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN);
-#endif
-  return;
-}
-
-void
-write_lock_binding(void)
-{
-  int status;
-
-  status = lock_reg(lockfd, F_SETLKW, F_WRLCK, 0, SEEK_SET, 0); 
-  if (0 != status) 
-    log_sys("set write lock");
-  return;
-}
-
-void 
-un_lock_read_binding(void)
-{
-  un_lock_write_binding();
-}
-
-void 
-un_lock_write_binding(void)
-{
-  int status;
-  
-  status = lock_reg(lockfd, F_SETLKW, F_UNLCK, 0, SEEK_SET, 0);
-  if (0 != status)
-    log_sys("unlock");
-  return;
-}
-#endif
-
 int
 get_entry(char *dom, struct binding *entry)
 {
@@ -765,15 +803,18 @@ update_entry(struct binding *entry)
     return;
   write_lock_binding();
   memcpy(ypdb, entry, sizeof (struct binding));
-#if !MMAP_SHARED_OK  
-  lseek(lockfd, 0, SEEK_SET);
-  writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN);
-#endif
+  if (!use_mmap)
+    {
+      lseek(lockfd, 0, SEEK_SET);
+      if (writen(lockfd, ypbindlist, sizeof(struct binding) * _MAXDOMAIN) <= 0)
+        log_sys("cannot write ypbindlist");
+    }
   un_lock_write_binding();
   if (debug)
     log_msg("%s entry for domain %s: server %s, port %d",
             entry->is_alive ? "updated" : "cleared", entry->domain,
-            inet_ntoa(entry->server_addr), ntohs(entry->server_port));
+            inet_ntoa(entry->server[entry->active].server_addr),
+            ntohs(entry->server[entry->active].server_port));
   return;
 }
 
