@@ -76,6 +76,9 @@ struct _dom_binding {
 	long int dom_vers;
 	int dom_lockfd;
 	int dom_alive;
+#ifdef __linux__
+	time_t timestamp; /* Last time seen alive */
+#endif
 	int dom_broadcast_pid;
 	int dom_pipe_fds[2];
 	int dom_default;
@@ -85,9 +88,13 @@ struct _dom_binding {
 #define WRITEFD ypdb->dom_pipe_fds[1]
 #define BROADFD broad_domain->dom_pipe_fds[1]
 
+#ifdef __linux__
+extern int daemon __P((int, int));
+#else
 extern bool_t xdr_domainname(), xdr_ypbind_resp();
 extern bool_t xdr_ypreq_key(), xdr_ypresp_val();
 extern bool_t xdr_ypbind_setdom();
+#endif
 
 void	checkwork __P((void));
 void	*ypbindproc_null_2 __P((SVCXPRT *, void *, CLIENT *));
@@ -153,7 +160,7 @@ SVCXPRT *transp;
 void *argp;
 CLIENT *clnt;
 {
-	static char res;
+	static char *res;
 
 	bzero((char *)&res, sizeof(res));
 	return (void *)&res;
@@ -162,7 +169,11 @@ CLIENT *clnt;
 struct ypbind_resp *
 ypbindproc_domain_2(transp, argp, clnt)
 SVCXPRT *transp;
+#ifdef __linux__
+char **argp;
+#else
 char *argp;
+#endif
 CLIENT *clnt;
 {
 	static struct ypbind_resp res;
@@ -174,13 +185,21 @@ CLIENT *clnt;
 	res.ypbind_respbody.ypbind_error = YPBIND_ERR_NOSERV;
 
 	for(ypdb=ypbindlist; ypdb; ypdb=ypdb->dom_pnext) {
+#ifdef __linux__
+		if( strcmp(ypdb->dom_domain, *argp) == 0)
+#else
 		if( strcmp(ypdb->dom_domain, argp) == 0)
+#endif
 			break;
 		}
 
 	if(ypdb==NULL) {
 		if (yp_restricted) {
+#ifdef __linux__
 			syslog(LOG_NOTICE, "Running in restricted mode -- request to bind domain \"%s\" rejected.\n", argp);
+#else
+			syslog(LOG_NOTICE, "Running in restricted mode -- request to bind domain \"%s\" rejected.\n", *argp);
+#endif
 			return &res;
 		}
 
@@ -197,7 +216,11 @@ CLIENT *clnt;
 			return &res;
 		}
 		bzero((char *)ypdb, sizeof *ypdb);
+#ifdef __linux__
+		strncpy(ypdb->dom_domain, *argp, sizeof ypdb->dom_domain);
+#else
 		strncpy(ypdb->dom_domain, argp, sizeof ypdb->dom_domain);
+#endif
 		ypdb->dom_vers = YPVERS;
 		ypdb->dom_alive = 0;
 		ypdb->dom_default = 0;
@@ -211,6 +234,17 @@ CLIENT *clnt;
 	}
 
 	if (ping(ypdb)) {
+#ifdef __linux__
+		/*
+		 * The Linux library will retry if it can't connect
+		 * to the server. We only return failure if we haven't
+		 * succeeded binding to a server for some time.
+		 *
+		 * This isn't perfect (the lib will print some timeout
+		 * errors) but at least it's transparent to the application.
+		 */
+		if (time(NULL) > ypdb->timestamp + FAIL_THRESHOLD * 2)
+#endif
 		return &res;
 	}
 
@@ -232,6 +266,8 @@ SVCXPRT *transp;
 struct ypbind_setdom *argp;
 CLIENT *clnt;
 {
+	static char *result = NULL;
+
 	struct sockaddr_in *fromsin, bindsin;
 
 	fromsin = svc_getcaller(transp);
@@ -240,7 +276,7 @@ CLIENT *clnt;
 	case YPSET_LOCAL:
 		if( fromsin->sin_addr.s_addr != htonl(INADDR_LOOPBACK)) {
 			svcerr_noprog(transp);
-			return;
+			return((void *) &result);
 		}
 		break;
 	case YPSET_ALL:
@@ -248,17 +284,17 @@ CLIENT *clnt;
 	case YPSET_NO:
 	default:
 		svcerr_noprog(transp);
-		return;
+		return((void *) &result);
 	}
 
 	if(ntohs(fromsin->sin_port) >= IPPORT_RESERVED) {
 		svcerr_noprog(transp);
-		return;
+		return((void *) &result);
 	}
 
 	if(argp->ypsetdom_vers != YPVERS) {
 		svcerr_noprog(transp);
-		return;
+		return((void *) &result);
 	}
 
 	bzero((char *)&bindsin, sizeof bindsin);
@@ -267,7 +303,7 @@ CLIENT *clnt;
 	bindsin.sin_port = argp->ypsetdom_port;
 	rpc_received(argp->ypsetdom_domain, &bindsin, 1);
 
-	return;
+	return((void *) &result);
 }
 
 static void
@@ -342,6 +378,16 @@ int sig;
 		children--;
 }
 
+#ifdef __linux__
+void gothit(sig)
+int sig;
+{
+	syslog(LOG_ERR, "got hit by signal %d - dying.\n");
+	signal(sig, SIG_DFL);
+	kill(getpid(), sig);
+}
+#endif
+
 void terminate(sig)
 int sig;
 {
@@ -362,12 +408,11 @@ int sig;
 	exit(0);
 }
 
-void
+int
 main(argc, argv)
 int argc;
 char **argv;
 {
-	char path[MAXPATHLEN];
 	struct timeval tv;
 	int i;
 	DIR *dird;
@@ -382,14 +427,14 @@ char **argv;
 	}
 
 	if(flock(yplockfd, LOCK_EX|LOCK_NB) == -1 && errno == EWOULDBLOCK) {
-		fprintf (stderr, "Another ypbind is already running. Aborting.\n");
+		fprintf (stderr, "ypbind: another ypbind is already running. Aborting.\n");
 		exit(1);
 	}
 
 	/* XXX domainname will be overriden if we use restricted mode */
 	yp_get_default_domain(&domainname);
-	if( domainname[0] == '\0') {
-		fprintf(stderr, "domainname not set. Aborting.\n");
+	if( domainname[0] == '\0' || strcmp(domainname, "(none)") == 0) {
+		fprintf(stderr, "ypbind: domainname not set. Aborting.\n");
 		exit(1);
 	}
 
@@ -430,24 +475,24 @@ char **argv;
 
 	udptransp = svcudp_create(RPC_ANYSOCK);
 	if (udptransp == NULL) {
-		fprintf(stderr, "cannot create udp service.\n");
+		fprintf(stderr, "ypbind: cannot create udp service.\n");
 		exit(1);
 	}
 	if (!svc_register(udptransp, YPBINDPROG, YPBINDVERS, ypbindprog_2,
 	    IPPROTO_UDP)) {
-		fprintf(stderr, "unable to register (YPBINDPROG, YPBINDVERS, udp).\n");
+		fprintf(stderr, "ypbind: unable to register (YPBINDPROG, YPBINDVERS, udp).\n");
 		exit(1);
 	}
 
 	tcptransp = svctcp_create(RPC_ANYSOCK, 0, 0);
 	if (tcptransp == NULL) {
-		fprintf(stderr, "cannot create tcp service.\n");
+		fprintf(stderr, "ypbind: cannot create tcp service.\n");
 		exit(1);
 	}
 
 	if (!svc_register(tcptransp, YPBINDPROG, YPBINDVERS, ypbindprog_2,
 	    IPPROTO_TCP)) {
-		fprintf(stderr, "unable to register (YPBINDPROG, YPBINDVERS, tcp).\n");
+		fprintf(stderr, "ypbind: unable to register (YPBINDPROG, YPBINDVERS, tcp).\n");
 		exit(1);
 	}
 
@@ -467,6 +512,15 @@ char **argv;
 
 	signal(SIGCHLD, reaper);
 	signal(SIGTERM, terminate);
+#ifdef __linux__
+	signal(SIGPIPE, gothit);
+	signal(SIGILL,  gothit);
+	signal(SIGBUS,  gothit);
+	signal(SIGFPE,  gothit);
+	signal(SIGPIPE, gothit);
+	signal(SIGALRM, gothit);
+	signal(SIGIO,   gothit);
+#endif
 
 	openlog(argv[0], LOG_PID, LOG_DAEMON);
 
@@ -499,6 +553,7 @@ char **argv;
 			break;
 		}
 	}
+	return 0;
 }
 
 void
@@ -617,6 +672,9 @@ struct _dom_binding *ypdb;
 {
 	bool_t out = FALSE;
 	enum clnt_stat stat;
+#ifdef __linux__
+	char *ypdomain;
+#endif
 
 	if (children >= MAX_CHILDREN || ypdb->dom_broadcast_pid)
 		return;
@@ -626,7 +684,7 @@ struct _dom_binding *ypdb;
 		return;
 	}
 
-	if (ypdb->dom_vers = -1 && (long)ypdb->dom_server_addr.sin_addr.s_addr)
+	if (ypdb->dom_vers == -1 && (long)ypdb->dom_server_addr.sin_addr.s_addr)
 		syslog(LOG_WARNING, "NIS server [%s] for domain \"%s\" not responding",
 		inet_ntoa(ypdb->dom_server_addr.sin_addr), ypdb->dom_domain);
 
@@ -658,9 +716,16 @@ struct _dom_binding *ypdb;
 
 	retries = 0;
 
+#ifdef __linux__
+	ypdomain = ypdb->dom_domain;
+	stat = clnt_broadcast(YPPROG, YPVERS, YPPROC_DOMAIN_NONACK,
+	    xdr_domainname, (void *)&ypdomain, xdr_bool, (char *)&out,
+	    broadcast_result);
+#else
 	stat = clnt_broadcast(YPPROG, YPVERS, YPPROC_DOMAIN_NONACK,
 	    xdr_domainname, (char *)ypdb->dom_domain, xdr_bool, (char *)&out,
 	    broadcast_result);
+#endif
 
 	if (stat != RPC_SUCCESS) {
 		bzero((char *)&ypdb->dom_server_addr,
@@ -693,7 +758,9 @@ struct _dom_binding *ypdb;
 	enum clnt_stat stat;
 	int rpcsock = RPC_ANYSOCK;
 	CLIENT *client_handle;
-	time_t t;
+#ifdef __linux__
+	char *ypdomain;
+#endif
 
 	interval.tv_sec = FAIL_THRESHOLD;
 	interval.tv_usec = 0;
@@ -707,22 +774,38 @@ struct _dom_binding *ypdb;
 		YPPROG, YPVERS, interval, &rpcsock, RPCSMALLMSGSIZE,
 		RPCSMALLMSGSIZE)) == (CLIENT *)NULL) {
 		/* Can't get a handle: we're dead. */
+#ifdef __linux__
+		if (ypdb->dom_alive) ypdb->timestamp = time(NULL);
+#endif
 		ypdb->dom_alive = 0;
 		ypdb->dom_vers = -1;
 		broadcast(ypdb);
 		return(1);
 	}
 
+#ifdef __linux__
+	ypdomain = ypdb->dom_domain;
+	if ((stat = clnt_call(client_handle, YPPROC_DOMAIN,
+		xdr_domainname, (void *)&ypdomain, xdr_bool,
+		(char *)&out, timeout)) != RPC_SUCCESS || out == FALSE) {
+#else
 	if ((stat = clnt_call(client_handle, YPPROC_DOMAIN,
 		xdr_domainname, (char *)ypdb->dom_domain, xdr_bool,
 		(char *)&out, timeout)) != RPC_SUCCESS || out == FALSE) {
+#endif
+
+#ifdef __linux__
+		if (ypdb->dom_alive) ypdb->timestamp = time(NULL);
+#endif
 		ypdb->dom_alive = 0;
 		ypdb->dom_vers = -1;
 		clnt_destroy(client_handle);
 		broadcast(ypdb);
 		return(1);
 	}
-
+#ifdef __linux__
+	ypdb->timestamp = time(NULL);
+#endif
 	clnt_destroy(client_handle);
 	return(0);
 }
@@ -813,7 +896,7 @@ int force;
 	}
 
 	/* We've recovered from a crash: inform the world. */
-	if (ypdb->dom_vers = -1 && ypdb->dom_server_addr.sin_addr.s_addr)
+	if (ypdb->dom_vers == -1 && ypdb->dom_server_addr.sin_addr.s_addr)
 		syslog(LOG_WARNING, "NIS server [%s] for domain \"%s\" OK",
 		inet_ntoa(raddrp->sin_addr), ypdb->dom_domain);
 
@@ -823,7 +906,9 @@ int force;
 	ypdb->dom_vers = YPVERS;
 	ypdb->dom_alive = 1;
 	ypdb->dom_broadcast_pid = 0;
-
+#ifdef __linux__
+	ypdb->timestamp = time(NULL);
+#endif
 	if(ypdb->dom_lockfd != -1)
 		close(ypdb->dom_lockfd);
 
