@@ -1,5 +1,7 @@
 /*
-** access.c 
+** access.c
+**
+** Copyright (C) 1997 Thorsten Kukuk
 **
 ** This file is part of the NYS YP Server.
 **
@@ -12,7 +14,7 @@
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU General Public
 ** License along with the NYS YP Server; see the file COPYING.  If
 ** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
@@ -21,7 +23,7 @@
 ** Author: Thorsten Kukuk <kukuk@uni-paderborn.de>
 */
 
-static char rcsid[] = "$Id: access.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
+static char rcsid[] = "$Id: access.c,v 1.7 1997/04/08 21:15:26 kukuk Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -32,6 +34,9 @@ static char rcsid[] = "$Id: access.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 
 #include <netdb.h>
 #include <syslog.h>
+#ifndef LOG_DAEMON
+#include <sys/syslog.h>
+#endif
 #include <string.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -41,6 +46,7 @@ static char rcsid[] = "$Id: access.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 
 #include "ypserv.h"
 #include "yp_msg.h"
+#include "yp_db.h"
 
 #ifndef HAVE_LIBWRAP
 
@@ -48,7 +54,7 @@ static char rcsid[] = "$Id: access.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include <stdlib.h>
 #include <memory.h>
 #include <sys/param.h>
-#include <sys/socket.h>
+/* #include <sys/socket.h> */
 
 #endif
 
@@ -60,18 +66,18 @@ void load_config(void)
 
   if(conf != NULL)
     {
-      yp_msg("Reloading %s/ypserv.conf\n",ETCDIR);
+      yp_msg("Reloading %s/ypserv.conf\n",CONFDIR);
       while(conf)
 	{
 	  tmp = conf;
 	  conf=conf->next;
-	  
+
 	  free(tmp->map);
 	  free(tmp);
 	}
     }
 
-  conf = load_ypserv_conf(ETCDIR);
+  conf = load_ypserv_conf(CONFDIR);
 }
 
 #ifndef HAVE_LIBWRAP
@@ -80,7 +86,7 @@ void load_config(void)
 #define SECURENETS "securenets"
 #endif
 
-typedef struct securenet 
+typedef struct securenet
 {
   struct in_addr netmask;
   struct in_addr network;
@@ -96,9 +102,9 @@ void load_securenets(void)
   securenet_t *work, *tmp;
   int line = 0;
 
-  /* 
+  /*
    * If securenets isn't NULL, we should reload the securents
-   * file. 
+   * file.
    */
   if(securenets != NULL)
     {
@@ -110,7 +116,7 @@ void load_securenets(void)
 	  free(work);
 	}
     }
-  securenets = NULL;  
+  securenets = NULL;
   work = NULL;
   tmp = NULL;
 
@@ -166,7 +172,7 @@ void load_securenets(void)
 	  free(tmp);
 	  continue;
 	}
-      
+
 #if defined(HAVE_INET_ATON)
       if(!inet_aton(buf2, &tmp->network))
 #else
@@ -189,7 +195,7 @@ void load_securenets(void)
 	  work=work->next;
 	}
     }
-  fclose(in);  
+  fclose(in);
 
   if(debug_flag)
     {
@@ -214,7 +220,7 @@ int securenet_host(const struct in_addr sin_addr)
    else
      while(ptr != NULL)
        {
-	 if((ptr->netmask.s_addr & sin_addr.s_addr) == 
+	 if((ptr->netmask.s_addr & sin_addr.s_addr) ==
 	    ptr->network.s_addr)
 	   return 1;
 	 ptr = ptr->next;
@@ -253,15 +259,15 @@ static char *ypproc_name(int proc)
 
 /*
 ** The is_valid_domain function checks the domain specified bze the
-** caller to make sure it's actually served by this server. 
-** 
+** caller to make sure it's actually served by this server.
+**
 ** Return 1 if the name is a valid domain name served by us, else 0.
 */
 
 int is_valid_domain(const char *domain)
 {
     struct stat sbuf;
-    
+
 
     if (domain == NULL ||
 	strcmp(domain, "binding") == 0 ||
@@ -269,7 +275,7 @@ int is_valid_domain(const char *domain)
 	strcmp(domain, ".") == 0 ||
 	strchr(domain, '/'))
 	return 0;
-    
+
     if (stat(domain, &sbuf) < 0 || !S_ISDIR(sbuf.st_mode))
 	return 0;
 
@@ -281,21 +287,26 @@ int is_valid_domain(const char *domain)
 ** is secure. But if you have the libwrap.a library, you could use
 ** them, too. It's much more flexible.
 **
-** Return 1 if request comes from an authorized host
+** Return 1, if request comes from an authorized host
+** Return 0, if request comes from an unauthorized host
+** Return -1, if the domain is not valid
 */
 
-int is_valid_host(struct svc_req *rqstp, const char *map)
+int is_valid_host(struct svc_req *rqstp, const char *map, const char *domain)
 {
 #if HAVE_LIBWRAP
   extern int hosts_ctl(char *, char *, char *, char *);
 #endif
   struct sockaddr_in *sin;
   int status;
-  static long oldaddr=0;	/* so we dont log multiple times */
+  static unsigned long int oldaddr=0;	/* so we dont log multiple times */
   static int oldstatus=-1;
   char *h=NULL;
   struct hostent *hp;
 
+  if((domain != NULL) && (is_valid_domain(domain) == 0))
+    return -1;
+
   sin = svc_getcaller(rqstp->rq_xprt);
 
   /*
@@ -304,7 +315,7 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
   */
   if(tryresolve_flag)
     {
-      hp = _dns_gethostbyaddr((char *) &sin->sin_addr.s_addr,
+      hp = res_gethostbyaddr((char *) &sin->sin_addr.s_addr,
 			      sizeof (sin->sin_addr.s_addr), AF_INET);
       h = (hp && hp->h_name) ? hp->h_name : NULL;
     }
@@ -312,8 +323,8 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
 #ifndef FROM_UNKNOWN
 #define FROM_UNKNOWN STRING_UNKNOWN
 #endif
-  
-#ifdef HAVE_LIBWRAP 
+
+#ifdef HAVE_LIBWRAP
   status = hosts_ctl(progname,
 		     h?h:FROM_UNKNOWN,
 		     inet_ntoa(sin->sin_addr),
@@ -322,7 +333,7 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
   status = securenet_host(sin->sin_addr);
 #endif
 
-  if((map != NULL) && (conf != NULL) && status)
+  if((map != NULL) && status)
     {
       conffile_t *work;
 
@@ -334,9 +345,22 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
 	      break;
 	  work=work->next;
 	}
+
       if(work != NULL)
 	switch(work->security)
 	  {
+	  case SEC_DES:
+	    if (rqstp->rq_cred.oa_flavor != AUTH_DES)
+	      {
+		if(debug_flag)
+		  yp_msg("No AUTH_DES !\n");
+		svcerr_weakauth(rqstp->rq_xprt);
+		status = 0;
+	      }
+	    else
+	      if(work->mangle)
+		status=200+work->field;
+	    break;
 	  case SEC_NONE:
 	    if(work->mangle)
 	      status=200+work->field;
@@ -352,6 +376,26 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
 		status = 200+work->field;
 	    break;
 	  }
+      else
+	if(domain != NULL)
+	  {
+	    /*
+	    ** The map is not in the access list, maybe it
+	    ** has a YP_SECURE key ?
+	    */
+	    DB_FILE dbp = ypdb_open(domain, map);
+	    if (dbp != NULL)
+	      {
+		datum key;
+
+		key.dsize = sizeof("YP_SECURE")-1;
+		key.dptr = "YP_SECURE";
+		if(ypdb_exists(dbp, key))
+		  if(ntohs(sin->sin_port) >= IPPORT_RESERVED)
+		    status=0;
+		ypdb_close(dbp);
+	      }
+	  }
     }
 
   if(debug_flag)
@@ -370,7 +414,7 @@ int is_valid_host(struct svc_req *rqstp, const char *map)
     }
   oldaddr=sin->sin_addr.s_addr;
   oldstatus=status;
-    
+
   return status;
-  
+
 }
