@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 1996 Thorsten Kukuk 
+** Copyright (c) 1996, 1997 Thorsten Kukuk 
 **
 ** This file is part of the NYS YP Server.
 **
@@ -21,7 +21,9 @@
 ** Author: Thorsten Kukuk <kukuk@uni-paderborn.de>
 */
 
-static char rcsid[] = "$Id: yppush.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
+#ifndef LINT
+static const char rcsid[] = "$Id: yppush.c,v 1.8 1997/01/29 19:16:29 kukuk Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -34,9 +36,12 @@ static char rcsid[] = "$Id: yppush.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include <unistd.h>
 #include <signal.h>
 #include <rpc/rpc.h>
+#include <time.h>
 #include "yp.h"
 #include <rpcsvc/ypclnt.h>
+#include <arpa/inet.h>
 #include <sys/param.h>
+#include <sys/socket.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
 #include <ctype.h>
@@ -44,6 +49,7 @@ static char rcsid[] = "$Id: yppush.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include <errno.h>
 #include <gdbm.h>
 #include <string.h>
+#include <memory.h>
 #if defined(HAVE_GETOPT_H) && defined(HAVE_GETOPT_LONG)
 #include <getopt.h>
 #else
@@ -54,6 +60,10 @@ static char rcsid[] = "$Id: yppush.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include "yp_msg.h"
 #include "yppush.h"
 
+#ifndef HAVE_STRDUP
+#include <compat/strdup.c>
+#endif
+
 #ifndef HAVE_GETOPT_LONG
 #include <compat/getopt.c>
 #include <compat/getopt1.c>
@@ -118,6 +128,120 @@ int _rpc_dtablesize()
 }
 #endif
 
+static char *yppush_err_string(enum yppush_status status) 
+{
+  switch(status) 
+    {
+    case YPPUSH_SUCC:      return "Success";
+    case YPPUSH_AGE:       return "Master's version not newer";
+    case YPPUSH_NOMAP:     return "Can't find server for map";
+    case YPPUSH_NODOM:     return "Domain not supported";
+    case YPPUSH_RSRC:      return "Local resource alloc failure";
+    case YPPUSH_RPC:       return "RPC failure talking to server";
+    case YPPUSH_MADDR:     return "Can't get master address";
+    case YPPUSH_YPERR:     return "YP server/map db error";
+    case YPPUSH_BADARGS:   return "Request arguments bad";
+    case YPPUSH_DBM:       return "Local dbm operation failed";
+    case YPPUSH_FILE:      return "Local file I/O operation failed";
+    case YPPUSH_SKEW:      return "Map version skew during transfer";
+    case YPPUSH_CLEAR:     return "Can't send \"Clear\" req to local ypserv";
+    case YPPUSH_FORCE:     return "No local order number in map  use -f flag.";
+    case YPPUSH_XFRERR:    return "ypxfr error";
+    case YPPUSH_REFUSED:   return "Transfer request refused by ypserv";
+    }
+  return "YPPUSH: Unknown Error, this should not happen!";
+}
+
+void *yppushproc_null_1_svc(void *req, struct svc_req * rqstp)
+{
+  static void *resp = NULL;
+
+  if(verbose_flag > 1)
+    yp_msg("yppushproc_null_1_svc\n");
+
+  return &resp;
+}
+
+yppushresp_xfr *yppushproc_xfrresp_1_svc(void *req, struct svc_req * rqstp)
+{
+  static yppushresp_xfr resp;
+  struct sockaddr_in *sin;
+  char *h;
+  struct hostent *hp;
+  
+  if(verbose_flag > 1)
+    yp_msg("yppushproc_xfrresp_1_svc\n");
+
+  sin = svc_getcaller(rqstp->rq_xprt);
+  
+  hp = gethostbyaddr((char *) &sin->sin_addr.s_addr,
+                     sizeof (sin->sin_addr.s_addr), AF_INET);
+  h = (hp && hp->h_name) ? hp->h_name : inet_ntoa(sin->sin_addr);
+
+  memcpy(&resp, (yppushresp_xfr*)req, sizeof(yppushresp_xfr));
+  if(verbose_flag)
+    {
+      yp_msg("Status received from ypxfr on %s:\n",h);
+      yp_msg("\tTransfer %sdone: %s\n",resp.status==YPPUSH_SUCC?"":"not ",
+             yppush_err_string(resp.status));
+    }
+  else
+    if (resp.status!=YPPUSH_SUCC)
+      yp_msg("%s: %s\n", h, yppush_err_string(resp.status));
+
+  return &resp;
+}
+
+void yppush_xfrrespprog_1( struct svc_req *rqstp, SVCXPRT *transp)
+{
+  union {
+    int fill;
+  } argument;
+  static char *result;
+  static bool_t (*xdr_argument)(XDR *, void *), (*xdr_result)(XDR *, void *);
+  static char *(*local)( void *, struct svc_req *);
+ 
+  if(verbose_flag > 1)
+    yp_msg("yppush_xfrrespprog_1\n");
+ 
+  switch (rqstp->rq_proc) 
+    {
+    case YPPUSHPROC_NULL:
+      xdr_argument = (bool_t (*)(XDR *, void *))xdr_void;
+      xdr_result = (bool_t (*)(XDR *, void *))xdr_void;
+      local = (char *(*)( void *, struct svc_req *)) yppushproc_null_1_svc;
+      break;
+      
+    case YPPUSHPROC_XFRRESP:
+      xdr_argument = (bool_t (*) (XDR *, void *))xdr_yppushresp_xfr;
+      xdr_result = (bool_t (*)(XDR *, void *))xdr_void;
+      local = (char *(*)( void *, struct svc_req *)) yppushproc_xfrresp_1_svc;
+      break;
+      
+    default:
+      svcerr_noproc(transp);
+      exit(1);
+    }
+  memset(&argument, '\0',  sizeof(argument));
+  if (!svc_getargs(transp, xdr_argument, (void *) &argument)) 
+    {
+      svcerr_decode(transp);
+      exit(1);
+    }
+  result = (*local)(&argument, rqstp);
+  if (result != NULL && 
+      !svc_sendreply(transp, (xdrproc_t) xdr_result, result)) 
+    {
+      svcerr_systemerr(transp);
+    }
+  if (!svc_freeargs(transp, xdr_argument, (void *) &argument)) 
+    {
+      yp_msg("unable to free arguments\n");
+      exit(1);
+    }
+  exit(0);
+}
+
 static void yppush_svc_run(char *target)
 {
 #ifdef FD_SETSIZE
@@ -317,7 +441,7 @@ static int yppush_foreach(const char *host)
   char server[YPMAXPEER+2];
   int sock;
   struct sigaction sa;
-#if HAVE_WAIT3
+#if HAVE_WAIT4
   struct rusage r;
 #endif
 
@@ -408,7 +532,7 @@ static int yppush_foreach(const char *host)
 	  clnt_perror(PushClient, "YPPUSH: Cannot call YPPROC_XFR");
 	  kill(transid, SIGTERM);
 	}
-#if HAVE_WAIT3
+#if HAVE_WAIT4
       wait4(transid, &sock, 0, &r);
 #else
       waitpid(transid,&sock, 0);
@@ -425,7 +549,8 @@ static int yppush_foreach(const char *host)
   return 0;
 }
 	
-static void sig_child(int sig)
+static void 
+sig_child (int sig)
 {
    int status;
 
@@ -441,7 +566,8 @@ static void sig_child(int sig)
      }
 }
 
-static inline void Usage(int exit_code)
+static inline void 
+Usage (int exit_code)
 {
   yp_msg("Usage: yppush [-d domain] [-t timeout] [-p #] [-h host] [-v] mapname ...\n");
   yp_msg("       yppush --version\n");
@@ -543,7 +669,11 @@ int main(int argc, char **argv)
   if(hostliste == NULL)
     {
       f.foreach=add_slave_server;
-      y=yp_all(DomainName, "ypservers", &f);
+#ifdef OSF_KLUDGE
+      y = yp_all (DomainName, "ypservers", f);
+#else
+      y = yp_all (DomainName, "ypservers", &f);
+#endif
       if (y && y!=YP_NOMORE) 
 	{
 	  yp_msg("Could not read ypservers: %d %s\n",
