@@ -1,6 +1,7 @@
 /*
-** ypserv.c                     YP server main routines.
+** ypserv.c - YP server main routines.
 **
+** Copyright (c) 1996, 1997 Thorsten Kukuk
 ** Copyright (c) 1993 Signum Support AB, Sweden
 **
 ** This file is part of the NYS YP Server.
@@ -14,7 +15,7 @@
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU General Public
 ** License along with the NYS YP Server; see the file COPYING.  If
 ** not, write to the Free Software Foundation, Inc., 675 Mass Ave,
@@ -24,7 +25,9 @@
 ** Modified: Thorsten Kukuk <kukuk@uni-paderborn.de>
 */
 
-static char rcsid[] = "$Id: ypserv.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
+#ifndef LINT
+static const char rcsid[] = "$Id: ypserv.c,v 1.5 1997/04/08 21:20:50 kukuk Exp $";
+#endif
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -41,6 +44,9 @@ static char rcsid[] = "$Id: ypserv.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include <memory.h>
 #include <unistd.h>
 #include <syslog.h>
+#ifndef LOG_DAEMON
+#include <sys/syslog.h>
+#endif
 #include <signal.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -66,6 +72,10 @@ static char rcsid[] = "$Id: ypserv.c,v 1.0.4.1 1996/07/14 15:10:45 kukuk Exp $";
 #include "yp_msg.h"
 #include "ypserv.h"
 
+#ifndef SA_RESTART
+#define SA_RESTART 0
+#endif
+
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
@@ -93,7 +103,7 @@ extern int _rpc_dtablesize(void);
 int _rpc_dtablesize()
 {
         static int size;
-        
+
         if (size == 0) {
                 size = getdtablesize();
         }
@@ -140,33 +150,33 @@ ypprog_2(struct svc_req *rqstp, register SVCXPRT *transp)
   char *result;
   xdrproc_t xdr_argument, xdr_result;
   char *(*local)(char *, struct svc_req *);
-  
-  switch (rqstp->rq_proc) 
+
+  switch (rqstp->rq_proc)
     {
     case YPPROC_NULL:
       xdr_argument = (xdrproc_t) xdr_void;
       xdr_result = (xdrproc_t) xdr_void;
       local = (char *(*)(char *, struct svc_req *)) ypproc_null_2_svc;
       break;
-      
+
     case YPPROC_DOMAIN:
       xdr_argument = (xdrproc_t) xdr_domainname;
       xdr_result = (xdrproc_t) xdr_bool;
       local = (char *(*)(char *, struct svc_req *)) ypproc_domain_2_svc;
       break;
-      
+
     case YPPROC_DOMAIN_NONACK:
       xdr_argument = (xdrproc_t) xdr_domainname;
       xdr_result = (xdrproc_t) xdr_bool;
       local = (char *(*)(char *, struct svc_req *)) ypproc_domain_nonack_2_svc;
       break;
-      
+
     case YPPROC_MATCH:
       xdr_argument = (xdrproc_t) xdr_ypreq_key;
       xdr_result = (xdrproc_t) xdr_ypresp_val;
       local = (char *(*)(char *, struct svc_req *)) ypproc_match_2_svc;
       break;
-      
+
     case YPPROC_FIRST:
 #if 0 /* Bug in Sun's yp.x RPC prototype file */
       xdr_argument = (xdrproc_t) xdr_ypreq_key;
@@ -176,65 +186,68 @@ ypprog_2(struct svc_req *rqstp, register SVCXPRT *transp)
       xdr_result = (xdrproc_t) xdr_ypresp_key_val;
       local = (char *(*)(char *, struct svc_req *)) ypproc_first_2_svc;
       break;
-      
+
     case YPPROC_NEXT:
       xdr_argument = (xdrproc_t) xdr_ypreq_key;
       xdr_result = (xdrproc_t) xdr_ypresp_key_val;
       local = (char *(*)(char *, struct svc_req *)) ypproc_next_2_svc;
       break;
-      
+
     case YPPROC_XFR:
       xdr_argument = (xdrproc_t) xdr_ypreq_xfr;
       xdr_result = (xdrproc_t) xdr_ypresp_xfr;
       local = (char *(*)(char *, struct svc_req *)) ypproc_xfr_2_svc;
       break;
-      
+
     case YPPROC_CLEAR:
       xdr_argument = (xdrproc_t) xdr_void;
       xdr_result = (xdrproc_t) xdr_void;
       local = (char *(*)(char *, struct svc_req *)) ypproc_clear_2_svc;
       break;
-      
+
     case YPPROC_ALL:
       xdr_argument = (xdrproc_t) xdr_ypreq_nokey;
       xdr_result = (xdrproc_t) xdr_ypresp_all;
       local = (char *(*)(char *, struct svc_req *)) ypproc_all_2_svc;
       break;
-      
+
     case YPPROC_MASTER:
       xdr_argument = (xdrproc_t) xdr_ypreq_nokey;
       xdr_result = (xdrproc_t) xdr_ypresp_master;
       local = (char *(*)(char *, struct svc_req *)) ypproc_master_2_svc;
       break;
-      
+
     case YPPROC_ORDER:
       xdr_argument = (xdrproc_t) xdr_ypreq_nokey;
       xdr_result = (xdrproc_t) xdr_ypresp_order;
       local = (char *(*)(char *, struct svc_req *)) ypproc_order_2_svc;
       break;
-      
+
     case YPPROC_MAPLIST:
       xdr_argument = (xdrproc_t) xdr_domainname;
       xdr_result = (xdrproc_t) xdr_ypresp_maplist;
       local = (char *(*)(char *, struct svc_req *)) ypproc_maplist_2_svc;
       break;
-      
+
     default:
+      if (debug_flag)
+	yp_msg("Unknown YPPROC call!\n");
       svcerr_noproc(transp);
       return;
     }
-  (void) memset((char *)&argument, 0, sizeof (argument));
-  if (!svc_getargs(transp, xdr_argument, (caddr_t) &argument)) 
+
+  memset((char *)&argument, 0, sizeof (argument));
+  if (!svc_getargs(transp, xdr_argument, (caddr_t) &argument))
     {
       svcerr_decode(transp);
       return;
     }
   result = (*local)((char *)&argument, rqstp);
-  if (result != NULL && !svc_sendreply(transp, xdr_result, result)) 
+  if (result != NULL && !svc_sendreply(transp, xdr_result, result))
     {
       svcerr_systemerr(transp);
     }
-  if (!svc_freeargs(transp, xdr_argument, (caddr_t) &argument)) 
+  if (!svc_freeargs(transp, xdr_argument, (caddr_t) &argument))
     {
       yp_msg("unable to free arguments");
       exit(1);
@@ -251,10 +264,10 @@ void ypserv_svc_run(void)
 #endif /* def FD_SETSIZE */
   extern int forked;
   int pid;
-  
+
   /* Establish the identity of the parent ypserv process. */
   pid = getpid();
-  
+
   for (;;) {
 #ifdef FD_SETSIZE
     readfds = svc_fdset;
@@ -262,10 +275,10 @@ void ypserv_svc_run(void)
     readfds = svc_fds;
 #endif /* def FD_SETSIZE */
     switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
-		   (struct timeval *)0)) 
+		   (struct timeval *)0))
       {
       case -1:
-	if (errno == EINTR) 
+	if (errno == EINTR)
 	    continue;
 	syslog(LOG_ERR, "svc_run: - select failed: %m");
 	return;
@@ -288,8 +301,8 @@ void create_pidfile(void)
   pid_t pid;
   char pbuf[10], *ptr;
   struct flock lock;
-  
-  fd = open(_YPSERV_PIDFILE, O_CREAT | O_RDWR, 
+
+  fd = open(_YPSERV_PIDFILE, O_CREAT | O_RDWR,
 	    S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
   if (fd < 0)
     {
@@ -330,7 +343,7 @@ void create_pidfile(void)
   sprintf(pbuf, "%d\n", getpid());
   left = strlen(pbuf);
   ptr = pbuf;
-  while (left > 0) 
+  while (left > 0)
     {
       if ((written = write(fd, ptr, left)) <= 0)
 	return;               /* error */
@@ -348,9 +361,9 @@ void sig_child(int sig)
 {
    int st;
 
-#ifdef HAVE_WAIT3   
+#ifdef HAVE_WAIT3
    while (wait3(&st, WNOHANG, NULL) > 0)
-#else 
+#else
    while (waitpid(-1, &st, WNOHANG) > 0)
 #endif
       children--;
@@ -418,7 +431,7 @@ int main(int argc, char **argv)
 
     for (i = 1; i < argc && argv[i][0] == '-'; i++)
       {
-	if (strcmp( argv[i], "--version") == 0 || 
+	if (strcmp( argv[i], "--version") == 0 ||
 	    strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "-v") == 0)
 	  version_flag = 1;
 	else if (strcmp(argv[i], "-debug") == 0 || strcmp(argv[i], "-d") == 0)
@@ -462,13 +475,13 @@ int main(int argc, char **argv)
     else
     {
 	int i;
-	
+
 	if (fork())
 	    exit(0);
 
 	for (i = 0; i < 255; i++)
 	    close(i);
-	
+
 	if (fork())
 	    exit(0);
     }
@@ -488,7 +501,7 @@ int main(int argc, char **argv)
     }
 
     create_pidfile();
-    
+
 #ifndef HAVE_LIBWRAP
     load_securenets();
 #endif
@@ -502,10 +515,10 @@ int main(int argc, char **argv)
     sa.sa_handler = SIG_IGN;
 #if !defined(sun) || (defined(sun) && defined(__svr4__))
     sa.sa_flags |= SA_RESTART;
-    /* 
+    /*
      * The opposite to SA_ONESHOT, do  not  restore
      * the  signal  action.  This provides behavior
-     * compatible with BSD signal semantics. 
+     * compatible with BSD signal semantics.
      */
 #endif
     sigemptyset(&sa.sa_mask);
@@ -553,7 +566,7 @@ int main(int argc, char **argv)
     (void) pmap_unset(YPPROG, YPVERS);
     if(sunos_kludge_flag)
       (void) pmap_unset(YPPROG, 1);
-    
+
     if (my_port >= 0)
     {
 	my_socket = socket (AF_INET, SOCK_DGRAM, 0);
@@ -563,6 +576,7 @@ int main(int argc, char **argv)
 	    exit (1);
 	}
 
+	memset((char *) &socket_address, 0, sizeof(socket_address));
 	socket_address.sin_family = AF_INET;
 	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
 	socket_address.sin_port = htons (my_port);
@@ -602,7 +616,7 @@ int main(int argc, char **argv)
 	  yp_msg("unable to register (YPPROG, 1, udp).");
 	  exit(1);
 	}
-    
+
     if (my_port >= 0)
     {
 	my_socket = socket (AF_INET, SOCK_STREAM, 0);
@@ -613,6 +627,7 @@ int main(int argc, char **argv)
 	    exit (1);
 	}
 
+	memset((char *) &socket_address, 0, sizeof(socket_address));
 	socket_address.sin_family = AF_INET;
 	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
 	socket_address.sin_port = htons (my_port);
